@@ -89,6 +89,8 @@ CREATE TABLE device_os_types (
 id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
 name VARCHAR(64) NOT NULL,
 dev_type INT DEFAULT NULL,
+directory_name VARCHAR(64) DEFAULT NULL,
+file_extension VARCHAR(32) DEFAULT NULL,
 FOREIGN KEY (dev_type) REFERENCES device_types(id)
 );
 
@@ -285,6 +287,12 @@ FOREIGN KEY (key_name) REFERENCES config_keys(id),
 FOREIGN KEY (profile_id) REFERENCES config_profiles(id) ON DELETE CASCADE
 );
 
+DROP TABLE IF EXISTS device_models;
+CREATE TABLE device_models ( -- holds different device models description, daemon - check + write once, GUI for display only, never delete
+id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
+model VARCHAR(128)
+);
+
 DROP TABLE IF EXISTS devices;
 CREATE TABLE devices ( -- main devices table, gathering basic information regarding devices
 id_device SMALLINT UNSIGNED PRIMARY KEY, -- device numeric ID (1-65536)
@@ -301,6 +309,7 @@ serno VARCHAR(32) DEFAULT NULL,
 active_sim VARCHAR(32) DEFAULT NULL, -- text from device saying which SIM card is active
 fwfname INT NOT NULL DEFAULT 1,
 hostname VARCHAR(64) DEFAULT NULL,
+model INT DEFAULT NULL,
 snmpname VARCHAR(64) DEFAULT NULL,
 hostname_desired VARCHAR(64) DEFAULT NULL,
 config_profile INT DEFAULT NULL, -- this is governing profile, 0 (NULL) by default
@@ -321,13 +330,28 @@ FOREIGN KEY (automatic_profile) REFERENCES config_profiles(id),
 FOREIGN KEY (fwfname) REFERENCES fw_firmware_names(id),
 FOREIGN KEY (agent_version) REFERENCES agent_versions(id),
 FOREIGN KEY (agent_profile) REFERENCES agent_profiles(id),
-FOREIGN KEY (auth_profile) REFERENCES auth_profiles(id)
+FOREIGN KEY (auth_profile) REFERENCES auth_profiles(id),
+FOREIGN KEY (model) REFERENCES device_models(id)
 );
 
 ALTER TABLE config_profiles ADD FOREIGN KEY (automatic) REFERENCES devices(id_device) ON DELETE SET NULL;
 
 ALTER TABLE config_profiles ADD FOREIGN KEY (master) REFERENCES devices(id_device) ON DELETE SET NULL;
 
+DROP TABLE IF EXISTS snmp_devices;
+CREATE TABLE snmp_devices(
+id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
+address VARCHAR(128) UNIQUE DEFAULT NULL,
+community_string VARCHAR(64) DEFAULT NULL,
+protocol_version INT DEFAULT 2,
+last_success DATETIME DEFAULT NULL,
+last_failure DATETIME DEFAULT NULL,
+errors_since_success INT DEFAULT 0,
+registered_as SMALLINT UNSIGNED DEFAULT NULL,
+registered_as_devid VARCHAR(4) DEFAULT NULL,
+devtoken_ag_copy VARCHAR(32) DEFAULT NULL
+);
+
 DROP TABLE IF EXISTS sim_cards;
 CREATE TABLE sim_cards(
 id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
@@ -373,36 +397,22 @@ FOREIGN KEY (id_device) REFERENCES devices(id_device) ON DELETE CASCADE,
 FOREIGN KEY (module) REFERENCES user_modules(id)
 );
 
-DROP TABLE IF EXISTS tunnel_ssh;
-CREATE TABLE tunnel_ssh (
-id INT NOT NULL PRIMARY KEY AUTO_INCREMENT,
-id_device SMALLINT UNSIGNED DEFAULT NULL, -- id of the device where does the tunnel lead to
-user_port INT DEFAULT 0, -- port where the user is going to connect (SSH)
-device_port INT DEFAULT 0,
-request_time DATETIME DEFAULT NULL,
-setup_time DATETIME DEFAULT NULL,
-destroy_time DATETIME DEFAULT NULL, -- last-resort desctruction time
-last_transfer DATETIME DEFAULT NULL,
-status INT DEFAULT 0,
-requested_op INT DEFAULT 0,
-FOREIGN KEY (status) REFERENCES tunnel_status(id),
-FOREIGN KEY (id_device) REFERENCES devices(id_device) ON DELETE CASCADE,
-FOREIGN KEY (requested_op) REFERENCES tunnel_op(id)
-);
-
-DROP TABLE IF EXISTS tunnel_web; -- table for web tunnels (to HTTPS/443 port / localhost of the device)
-CREATE TABLE tunnel_web (
+DROP TABLE IF EXISTS tunnels; -- table for tunnels (WEB to HTTPS/443 SSH to 22 port / localhost of the device)
+CREATE TABLE tunnels (
 id INT NOT NULL PRIMARY KEY,
-id_device SMALLINT UNSIGNED DEFAULT NULL, -- id of the device where does the tunnel lead to
-url VARCHAR(32) DEFAULT NULL, -- part of the URL, the random one, e.g. https://white.beta.dwarftech.cz/webtunnels/<this_part> - AUTOCFG
-device_port INT DEFAULT 0, -- port which does the tunnel to the device listens at - AUTOCFG 127.0.0.1:<this_port>
-request_time DATETIME DEFAULT NULL, -- timestamp of the request, AUTOCFG
-setup_time DATETIME DEFAULT NULL, -- when the tunnel was set up AUTOCFG
+id_device SMALLINT UNSIGNED DEFAULT NULL, -- id of the device where does the tunnel lead to, INTERNAL
+url VARCHAR(128) DEFAULT NULL, -- the subdomain part of the URL, e.g. https://<this_part>.<regular DG server address>
+device_port INT DEFAULT 0, -- port which does the tunnel to the device listens at - AUTOCFG 127.0.0.1:<this_port>; INTERNAL
+request_time DATETIME DEFAULT NULL, -- timestamp of the request
+setup_time DATETIME DEFAULT NULL, -- when the tunnel was set up
 expected_time DATETIME DEFAULT NULL, -- when is the device expected to process the pending operation
 destroy_time DATETIME DEFAULT NULL, -- last-resort destruction time
-last_transfer DATETIME DEFAULT NULL, -- statistics AUTOCFG
-status INT DEFAULT 0, -- last known status of the tunnel, AUTOCFG
+last_transfer DATETIME DEFAULT NULL, -- statistics
+status INT DEFAULT 0, -- last known status of the tunnel
 requested_op INT DEFAULT 0, -- operation requested by user
+ssh BOOLEAN DEFAULT FALSE, -- is this an ssh tunnel request?
+proxy_url VARCHAR(128) DEFAULT NULL, -- information for the user how to set the SSH proxy. Useful for ssh tunnels only.
+password VARCHAR(16) DEFAULT NULL, -- if set, it is a password protecting access to the ssh web gateway (webssh or sshwifty)
 FOREIGN KEY (status) REFERENCES tunnel_status(id),
 FOREIGN KEY (id_device) REFERENCES devices(id_device) ON DELETE CASCADE,
 FOREIGN KEY (requested_op) REFERENCES tunnel_op(id)
@@ -432,6 +442,10 @@ root_avail INT UNSIGNED DEFAULT 0, -- available space on root FS - in MiBs
 loadavg1 DOUBLE DEFAULT 0, -- last minute average load
 loadavg5 DOUBLE DEFAULT 0, -- last 5 minutes average load
 loadavg15 DOUBLE DEFAULT 0, -- last 15 minutes average load
+sim_ipaddr VARCHAR(16) DEFAULT NULL,
+sim_ipmask SMALLINT UNSIGNED DEFAULT NULL,
+sim_ip6addr VARCHAR(48) DEFAULT NULL,
+sim_ip6pref SMALLINT UNSIGNED DEFAULT NULL,
 last_reboot TIMESTAMP DEFAULT CURRENT_TIMESTAMP, -- timestamp of last reboot
 reboot_counter INT UNSIGNED DEFAULT 0, -- total # of reboots
 signal_strength SMALLINT DEFAULT NULL, -- strength of signal, typically between -0 .. -110
@@ -900,7 +914,7 @@ FOREIGN KEY (oper_second) REFERENCES typetable_operators(id)
 DROP TABLE IF EXISTS product_conf;
 CREATE TABLE product_conf (
 id SMALLINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
-name ENUM ('ExternalURL', 'DB version', 'isFirstRun', 'EnabledEmailNotifications', 'EnableEmailNotificationToAddress', 'NotificationEmail', 'SlackToken', 'SlackChannel', 'SenderEmail', 'SmtpServer', 'SmtpPort', 'SmtpSecurity', 'SmtpLogin', 'SmtpPassword', 'ServiceCoreName', 'WebTunnelURL', 'SysctlPath', 'ProductRootDir', 'ProductSrvDir', 'SAASMode', 'LocalPort', 'AggregatedLogsFile', 'SupportLink', 'DevicesTableColumns') NOT NULL,
+name ENUM ('ExternalURL', 'DB version', 'isFirstRun', 'EnabledEmailNotifications', 'EnableEmailNotificationToAddress', 'NotificationEmail', 'SlackToken', 'SlackChannel', 'SenderEmail', 'SmtpServer', 'SmtpPort', 'SmtpSecurity', 'SmtpLogin', 'SmtpPassword', 'ServiceCoreName', 'WebTunnelURL', 'SysctlPath', 'ProductRootDir', 'ProductSrvDir', 'SAASMode', 'LocalPort', 'AggregatedLogsFile', 'SupportLink', 'DevicesTableColumns', 'SNMPGatewayThreads', 'SNMPGatewayBatch', 'SNMPGatewayTimeout', 'SNMPGatewayRetries', 'SNMPGatewayInterval', 'SNMPGatewayLoglevel', 'ServiceSNMPGatewayName', 'SNMPSSLOverride') NOT NULL,
 valtype SMALLINT UNSIGNED DEFAULT 1,
 system BOOLEAN DEFAULT false,
 intval INT,
@@ -911,6 +925,49 @@ time DATETIME DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
 FOREIGN KEY (valtype) REFERENCES typetable_value_types(id)
 );
 
+DELIMITER //
+CREATE TRIGGER upd_snmp_config BEFORE UPDATE ON product_conf
+FOR EACH ROW
+BEGIN
+  IF NEW.name = 'SNMPGatewayThreads' THEN BEGIN
+    IF NEW.intval > 32 THEN SET NEW.intval = 32;
+    ELSEIF NEW.intval < 1 THEN SET NEW.intval = 1;
+    END IF;
+    END;
+  ELSEIF NEW.name = 'SNMPGatewayBatch' THEN BEGIN
+    IF NEW.intval > 256 THEN SET NEW.intval = 256;
+    ELSEIF NEW.intval < 8 THEN SET NEW.intval = 8;
+    END IF;
+    END;
+  ELSEIF NEW.name = 'SNMPGatewayTimeout' THEN BEGIN
+    IF NEW.intval > 60 THEN SET NEW.intval = 60;
+    ELSEIF NEW.intval < 2 THEN SET NEW.intval = 2;
+    END IF;
+    END;
+  ELSEIF NEW.name = 'SNMPGatewayRetries' THEN BEGIN
+    IF NEW.intval > 6 THEN SET NEW.intval = 6;
+    ELSEIF NEW.intval < 1 THEN SET NEW.intval = 1;
+    END IF;
+    END;
+  ELSEIF NEW.name = 'SNMPGatewayInterval' THEN BEGIN
+    IF NEW.intval > 3600 THEN SET NEW.intval = 3600;
+    ELSEIF NEW.intval < 120 THEN SET NEW.intval = 120;
+    END IF;
+    END;
+  ELSEIF NEW.name = 'SNMPSSLOverride' THEN BEGIN
+    IF NEW.intval > 1 THEN SET NEW.intval = 1;
+    ELSEIF NEW.intval < -1 THEN SET NEW.intval = -1;
+    END IF;
+    END;
+  ELSEIF NEW.name = 'SNMPGatewayLoglevel' THEN BEGIN
+    IF NEW.intval > 50 THEN SET NEW.intval = 50;
+    ELSEIF NEW.intval < 15 THEN SET NEW.intval = 15;
+    END IF;
+    END;
+  END IF;
+END; //
+DELIMITER ;
+
 DROP TABLE IF EXISTS data_change_source;
 CREATE TABLE data_change_source (
 id SMALLINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
@@ -936,7 +993,10 @@ name VARCHAR(64) NOT NULL,
 internal_type SMALLINT UNSIGNED NOT NULL DEFAULT 3,
 nature SMALLINT UNSIGNED NOT NULL,
 context SMALLINT UNSIGNED NOT NULL,
-visible BOOLEAN DEFAULT true,
+visible BOOLEAN DEFAULT True,
+log_stream BOOLEAN DEFAULT True,
+log_db BOOLEAN DEFAULT True,
+log_file BOOLEAN DEFAULT True,
 FOREIGN KEY (internal_type) REFERENCES typetable_value_types(id),
 FOREIGN KEY (nature) REFERENCES data_atom_nature(id),
 FOREIGN KEY (context) REFERENCES data_atom_context(id)
@@ -957,3 +1017,12 @@ FOREIGN KEY (atom) REFERENCES data_atom(id),
 FOREIGN KEY (id_device) REFERENCES devices(id_device) ON DELETE CASCADE
 );
 
+CREATE TRIGGER delban_6_dcl
+BEFORE DELETE ON data_change_log
+FOR EACH ROW
+SIGNAL SQLSTATE '45000' SET MESSAGE_TEXT = 'Delete not allowed on simpletable';
+
+CREATE TRIGGER delban_7_device_models
+BEFORE DELETE ON device_models
+FOR EACH ROW
+SIGNAL SQLSTATE '45000' SET MESSAGE_TEXT = 'Delete not allowed on simpletable';
