@@ -6,8 +6,8 @@ namespace App\Controller\Admin\Ajax;
 
 use App\Entity\AgentProfile\AgentProfile;
 use App\Entity\ConfigProfile\ConfigProfile;
-use App\Entity\Device;
-use App\Entity\DeviceOsVersion;
+use App\Entity\Device\Device;
+use App\Entity\Device\DeviceOsVersion;
 use App\Repository\DeviceRepository;
 use App\Service\Api\AgentProfiles;
 use Doctrine\DBAL\ArrayParameterType;
@@ -20,18 +20,20 @@ use Symfony\Component\HttpFoundation\JsonResponse;
 use Symfony\Component\HttpFoundation\Request;
 use Symfony\Component\HttpFoundation\Response;
 use Symfony\Component\Routing\Annotation\Route;
+use App\Service\Api\Device as DeviceApi;
 
 #[Route(path: '/admin/ajax/device', name: 'ajax_device_')]
 class DeviceController extends Controller
 {
 
     /**
-     * Get devices not assigned to web tunnels
-     * @return JsonResponse|Response
+     * @param Request $request
+     * @param DeviceRepository $deviceRepository
+     * @return Response
      * @throws Exception
      */
-    #[Route(path: '/not-assigned-web-tunnel', name: 'get_not_web_tunnel', methods: ['GET'])]
-    public function getAllDevices(Request $request, DeviceRepository $deviceRepository): Response {
+    #[Route(path: '/not-assigned-tunnel', name: 'get_not_tunnel', methods: ['GET'])]
+    public function getDevicesNotAssignedTunnel(Request $request, DeviceRepository $deviceRepository): Response {
 
         if ($request->isXmlHttpRequest()) {
 
@@ -50,8 +52,11 @@ class DeviceController extends Controller
     }
 
     /**
-     * Get all Devices not in Monitoring Group.
+     * @param Request $request
+     * @param DeviceRepository $deviceRepository
+     * @param int $mntGroupId
      * @return Response
+     * @throws Exception
      */
     #[Route(path: '/get/all-not/{mntGroupId}', name: 'get_all_not_in_monitoring_group', methods: ['GET'])]
     public function getAllDevicesNotInMonitoringGroup(Request          $request,
@@ -76,10 +81,10 @@ class DeviceController extends Controller
     }
 
     /**
-     * Add Devices to Monitoring Groups
      * @param Request $request
      * @param ManagerRegistry $managerRegistry
      * @return Response
+     * @throws Exception
      */
     #[Route(path: '/add/monitoring-group', name: 'add_to_monitoring_group', methods: ['POST'])]
     public function addDevicesToMonitoringGroupAction(Request $request, ManagerRegistry $managerRegistry): Response
@@ -96,8 +101,9 @@ class DeviceController extends Controller
             $monitoringGroups = (array)$request->get('monitoringGroups');
             $selectedDevices = $request->get('selectedDevices');
 
-            if (!isset($monitoringGroups) || !isset($selectedDevices))
-                return new JsonResponse(['status' => 'Error'], 400);
+            if (!isset($monitoringGroups) || !isset($selectedDevices)) {
+				return new JsonResponse(['status' => 'Error'], 400);
+			}
 
             $deviceIds = [];
 
@@ -109,6 +115,10 @@ class DeviceController extends Controller
             $connection =  $managerRegistry->getConnection();
             $connection->beginTransaction();
 
+			if(empty($monitoringGroups)){
+				return new JsonResponse(['status' => 'Error', 'message' => 'No monitoring group given.'],400);
+			}
+
             foreach ($monitoringGroups as $monitoringGroupId) {
 
                 $placeholders = [];
@@ -279,12 +289,17 @@ class DeviceController extends Controller
             $fws = (array)$request->get('fws');
             $selectedDevices = $request->get('selectedDevices');
 
-            if (!isset($fws) || !isset($selectedDevices))
-                return new JsonResponse(['status' => 'Error'], 400);
+            if (!isset($fws) || !isset($selectedDevices)) {
+				return new JsonResponse(['status' => 'Error'], 400);
+			}
 
             $fws = $managerRegistry->getRepository(DeviceOsVersion::class)->findBy(['id' => $fws]);
             $deviceRepository = $managerRegistry->getRepository(Device::class);
 
+			if (!$fws){
+				return new JsonResponse(['status' => 'Error'], 400);
+			}
+
             $errors = [];
             $success = [];
             $values = [];
@@ -359,7 +374,9 @@ class DeviceController extends Controller
     }
 
     /**
-     * Add Agent Profile to Devices
+     * @param Request $request
+     * @param ManagerRegistry $managerRegistry
+     * @param AgentProfiles $agentProfilesApi
      * @return Response
      */
     #[Route(path: '/add/agent-profile', name: 'add_agent_profile', methods: ['POST'])]
@@ -377,44 +394,59 @@ class DeviceController extends Controller
             $agentProfiles = (array)$request->get('agentProfiles');
             $selectedDevices = $request->get('selectedDevices');
 
-            if (!isset($agentProfiles) && !isset($selectedDevices))
-                return new JsonResponse(['status' => 'Error'], 400);
+            if (!isset($agentProfiles) && !isset($selectedDevices)) {
+				return new JsonResponse(['status' => 'Error'], 400);
+			}
 
             $agentProfiles = $managerRegistry->getRepository(AgentProfile::class)->findBy(['id' => $agentProfiles]);
             $deviceRepository = $managerRegistry->getRepository(Device::class);
 
+			if (!$agentProfiles){
+				return new JsonResponse(['status' => 'Error'], 400);
+			}
+
             $errors = [];
             $success = [];
 
             foreach ($agentProfiles as $agentProfile) {
 
                 $deviceIds = [];
+				$deviceDevIds = [];
                 $agentProfileId = $agentProfile->getId();
 
                 foreach ($selectedDevices as $device) {
 
                     $device = $deviceRepository->findOneBy(['id' => $device['id']]);
+					if(!$device){
+						$deviceIds = [];
+						$errors[] = 'Invalid device given.';
+					}
                     $deviceIds[] = $device->getId();
+                    $deviceDevIds[] = $device->getDevId();
 
                 }
 
-                try {
+				if(!empty($deviceIds)){
 
-                    $result = $agentProfilesApi->assignDeviceAgentProfile($agentProfileId, $deviceIds);
-                    if($result->operation_result->Code < 0){
-                        throw new \Exception('Cannot call api');
-                    }
-                    /*$device->setAgentProfile($agentProfile);
-                    $managerRegistry->getManager()->persist($device);
-                    $managerRegistry->getManager()->flush();*/
+					try {
 
-                    $success[] = 'Selected profile was assigned.'; //$device->getDevId();
+						$result = $agentProfilesApi->assignDeviceAgentProfile($agentProfileId, $deviceIds);
+						if($result->operation_result->Code < 0){
+							throw new \Exception('Cannot call api');
+						}
+						/*$device->setAgentProfile($agentProfile);
+						$managerRegistry->getManager()->persist($device);
+						$managerRegistry->getManager()->flush();*/
 
-                } catch (\Exception $e) {
+						$success[] = implode(',', $deviceDevIds);
 
-                    $errors[] = 'Something bad happend.' . $e->getMessage(); //$device->getDevId();
+					} catch (\Exception $e) {
 
-                }
+						$errors[] = $e->getMessage(); //$device->getDevId();
+
+					}
+
+				}
 
             }
 
@@ -435,7 +467,9 @@ class DeviceController extends Controller
     }
 
     /**
-     * Get Devices with Config Profile
+     * @param Request $request
+     * @param DeviceRepository $deviceRepository
+     * @param ManagerRegistry $managerRegistry
      * @return Response
      * @throws Exception
      */
@@ -462,9 +496,9 @@ class DeviceController extends Controller
     }
 
     /**
-     * Get All Agent Profiles
+     * @param Request $request
+     * @param ManagerRegistry $managerRegistry
      * @return Response
-     * @throws Exception
      */
     #[Route(path: '/get/agent-profiles', name: 'get_agent_profiles', methods: ['GET'])]
     public function getAgentProfilesAction(Request $request, ManagerRegistry $managerRegistry): Response
@@ -489,8 +523,10 @@ class DeviceController extends Controller
     }
 
     /**
-     * Add Config Profile to Devices
+     * @param Request $request
+     * @param ManagerRegistry $managerRegistry
      * @return Response
+     * @throws Exception
      */
     #[Route(path: '/add/config-profile', name: 'add_config_profile', methods: ['POST'])]
     public function addConfigProfileToDeviceAction(Request $request, ManagerRegistry $managerRegistry): Response {
@@ -507,12 +543,17 @@ class DeviceController extends Controller
             $selectedDevices = $request->get('selectedDevices');
             $overwrite = $request->get('overwrite'); //retype to bool did not work for false
 
-            if (!isset($devices) && !isset($selectedDevices))
-                return new JsonResponse(['status' => 'Error'], 400);
+            if (!isset($devices) && !isset($selectedDevices)) {
+				return new JsonResponse(['status' => 'Error'], 400);
+			}
 
             $devices = $managerRegistry->getRepository(Device::class)->findBy(['id' => $devices]);
             $deviceRepository = $managerRegistry->getRepository(Device::class);
 
+			if (!$devices){
+				return new JsonResponse(['status' => 'Error'], 400);
+			}
+
             $errors = [];
             $success = [];
 
@@ -589,7 +630,9 @@ class DeviceController extends Controller
     }
 
     /**
-     * Remove Config Profile for Devices
+     * @param Request $request
+     * @param ManagerRegistry $managerRegistry
+     * @param DeviceRepository $deviceRepository
      * @return Response
      */
     #[Route(path: '/remove/config-profile', name: 'remove_config_profile', methods: ['POST'])]
@@ -657,4 +700,41 @@ class DeviceController extends Controller
         }
     }
 
+    #[Route(path: '/reboot/{deviceId}', name: 'reboot', methods: ['GET'])]
+    public function rebootAction(int $deviceId, Request $request, DeviceRepository $deviceRepository,
+                                 DeviceApi $deviceApi): Response
+    {
+        if (!$request->isXmlHttpRequest()) {
+
+            return new Response('Bad request.', 400);
+
+        }
+
+        $device = $deviceRepository->findBy(['id' => $deviceId]);
+
+        if(!$device){
+            return new JsonResponse(['status' => 'Error'], 400);
+        }
+
+        try{
+
+            $deviceApi->reboot([$deviceId]);
+
+        }catch (\Exception $e){
+
+            return new JsonResponse(['status' => 'Error', 'message' => $e->getMessage()],
+                400);
+
+        }
+
+        return new JsonResponse([
+            'code' => 200,
+            'message' => 'Device will be rebooted.',
+           /* 'success' => $success,
+            'errors' => $errors*/
+        ],
+            200);
+
+    }
+
 }
\ No newline at end of file
