@@ -8,15 +8,16 @@ use App\Controller\Admin\BaseController;
 use App\Entity\AgentProfile\AgentProfile;
 use App\Entity\Alert\Alert;
 use App\Entity\ConfigProfile\ConfigProfile;
-use App\Entity\Device;
-use App\Entity\DeviceOsVersion;
-use App\Entity\DeviceType;
-use App\Entity\Label;
-use App\Entity\LabelDeviceMap;
-use App\Entity\MonitoringGroup;
-use App\Entity\MonitoringGroupMapping;
-use App\Entity\SystemLabel;
-use App\Entity\SystemLabelDeviceMap;
+use App\Entity\Device\Device;
+use App\Entity\Snmp\Device as SnmpDevice;
+use App\Entity\Device\DeviceOsVersion;
+use App\Entity\Device\DeviceType;
+use App\Entity\Label\Label;
+use App\Entity\Label\LabelDeviceMap;
+use App\Entity\Label\SystemLabel;
+use App\Entity\Label\SystemLabelDeviceMap;
+use App\Entity\MonitoringGroup\MonitoringGroup;
+use App\Entity\MonitoringGroup\MonitoringGroupMapping;
 use App\Entity\Tool\AuthProfile;
 use App\Exception\AlertNotFoundException;
 use App\Exception\DeviceNotFoundException;
@@ -44,11 +45,13 @@ use App\Service\Api\ApiService;
 use App\Service\Api\DashboardLights;
 use App\Service\Command;
 use App\Service\DatatableService;
+use App\Service\ExportService;
 use App\Service\History;
 use Doctrine\DBAL\Exception;
 use Doctrine\ORM\EntityManagerInterface;
 use Doctrine\Persistence\ManagerRegistry;
 use Symfony\Component\DependencyInjection\ParameterBag\ParameterBagInterface;
+use Symfony\Component\Form\FormInterface;
 use Symfony\Component\HttpFoundation\BinaryFileResponse;
 use Symfony\Component\HttpFoundation\JsonResponse;
 use Symfony\Component\HttpFoundation\RedirectResponse;
@@ -56,7 +59,11 @@ use Symfony\Component\HttpFoundation\Request;
 use Symfony\Component\HttpFoundation\Response;
 use Symfony\Component\HttpFoundation\ResponseHeaderBag;
 use Symfony\Component\Routing\Annotation\Route;
+use Symfony\Component\Serializer\Encoder\CsvEncoder;
+use Symfony\Component\Serializer\Normalizer\ObjectNormalizer;
+use Symfony\Component\Serializer\Serializer;
 
+#[Route(path: '/admin/devices', name: 'admin_devices')]
 class DeviceController extends BaseController
 {
 
@@ -68,19 +75,25 @@ class DeviceController extends BaseController
      * @param DeviceRepository $deviceRepository
      * @param ProductFlagsRepository $productFlagsRepository
      */
-    public function __construct(ManagerRegistry  $managerRegistry, private readonly EntityManagerInterface $entityManager,
-                                private readonly DeviceRepository $deviceRepository, private readonly ProductFlagsRepository $productFlagsRepository) {
+    public function __construct(ManagerRegistry  $managerRegistry,
+                                private readonly EntityManagerInterface $entityManager,
+                                private readonly DeviceRepository $deviceRepository,
+                                private readonly ProductFlagsRepository $productFlagsRepository) {
 
         parent::__construct($managerRegistry);
 
     }
 
     /**
+     * @param Request $request
+     * @param DatatableResponse $responseService
+     * @param DatatableService $datatableService
      * @return JsonResponse|Response
      * @throws \Exception
      */
-    #[Route(path: 'admin/devices', name: 'admin_devices')]
-    public function listDevices(Request $request, DatatableResponse $responseService, DatatableService $datatableService){
+    #[Route(path: '', name: '')]
+    public function listDevices(Request $request, DatatableResponse $responseService, DatatableService $datatableService): JsonResponse|Response
+    {
 
         $isAjax = $request->isXmlHttpRequest();
         $this->filter = $request->get('column');
@@ -100,7 +113,7 @@ class DeviceController extends BaseController
             $qb = $datatableQueryBuilder->getQb();
             $qb->andWhere('vdevice.deleted = :deleted');
             $qb->setParameter('deleted', '0');
-            
+
             return $responseService->getResponse();
         }
 
@@ -113,12 +126,19 @@ class DeviceController extends BaseController
     }
 
     /**
+     * @param int $alertId
+     * @param Request $request
+     * @param ManagerRegistry $managerRegistry
+     * @param ApiService $apiService
+     * @param DatatableResponse $responseService
+     * @param DatatableService $datatableService
      * @return JsonResponse|Response
      * @throws AlertNotFoundException
      */
-    #[Route(path: 'admin/devices/alert/{alertId}', name: 'admin_devices_triggered_alert')]
+    #[Route(path: '/alert/{alertId}', name: '_triggered_alert')]
     public function listDevicesByAlertId(int $alertId, Request $request, ManagerRegistry $managerRegistry, ApiService $apiService,
-                                         DatatableResponse $responseService, DatatableService $datatableService){
+                                         DatatableResponse $responseService, DatatableService $datatableService): JsonResponse|Response
+    {
 
         $isAjax = $request->isXmlHttpRequest();
         $this->filter = $request->get('column');
@@ -161,13 +181,21 @@ class DeviceController extends BaseController
     }
 
     /**
+     * @param int $monitoringGroupId
+     * @param int $subsetId
+     * @param Request $request
+     * @param ManagerRegistry $managerRegistry
+     * @param DashboardLights $dashboardLightsApi
+     * @param DatatableResponse $responseService
+     * @param DatatableService $datatableService
      * @return JsonResponse|Response
      * @throws MonitoringGroupNotFoundException
      */
-    #[Route(path: 'admin/devices/dashboard/monitoring-group/{monitoringGroupId}/subset/{subsetId}', name: 'admin_devices_dashboard_light')]
+    #[Route(path: '/dashboard/monitoring-group/{monitoringGroupId}/subset/{subsetId}', name: '_dashboard_light')]
     public function listDevicesByDashboardLight(int $monitoringGroupId, int $subsetId, Request $request,
                                                 ManagerRegistry $managerRegistry, DashboardLights $dashboardLightsApi,
-                                         DatatableResponse $responseService, DatatableService $datatableService){
+                                         DatatableResponse $responseService, DatatableService $datatableService): JsonResponse|Response
+    {
 
         $isAjax = $request->isXmlHttpRequest();
         $this->filter = $request->get('column');
@@ -227,12 +255,19 @@ class DeviceController extends BaseController
     }
 
     /**
-     * @return JsonResponse|Response
+     * @param int $alertId
+     * @param Request $request
+     * @param ManagerRegistry $managerRegistry
+     * @param ApiService $apiService
+     * @param DatatableResponse $responseService
+     * @param DatatableService $datatableService
+     * @return JsonResponse|Response|null
      * @throws AlertNotFoundException
      */
-    #[Route(path: 'admin/devices/past-alert/{alertId}', name: 'admin_devices_past_alert')]
+    #[Route(path: '/past-alert/{alertId}', name: '_past_alert')]
     public function listDevicesByPastAlertId(int $alertId, Request $request, ManagerRegistry $managerRegistry, ApiService $apiService,
-                                         DatatableResponse $responseService, DatatableService $datatableService){
+                                         DatatableResponse $responseService, DatatableService $datatableService): JsonResponse|Response|null
+    {
 
         $isAjax = $request->isXmlHttpRequest();
         $this->filter = $request->get('column');
@@ -276,7 +311,38 @@ class DeviceController extends BaseController
     }
 
     /**
-     * @param $deviceId
+     * @throws \Exception
+     */
+    #[Route(path: '/data-download', name: '_data_download')]
+    public function dataDownload(ExportService $exportService): Response
+    {
+
+        $exportData = $exportService->getDataExportForDeviceDataTable();
+
+        $csvData = [];
+        $csvData[] = array_keys($exportData[0]);
+
+        foreach($exportData as $line){
+            $csvData[] = array_values($line);
+        }
+
+        $encoders = [new CsvEncoder(['no_headers' => true ])];
+        $normalizers = array(new ObjectNormalizer());
+        $serializer = new Serializer($normalizers, $encoders);
+        $csvContent = $serializer->serialize($csvData, 'csv');
+
+        $fileName = 'DeviceDwarfguardExport_' . date('d-m-Y_H-i');
+
+        $response = new Response($csvContent);
+        $response->headers->set('Content-Encoding', 'UTF-8');
+        $response->headers->set('Content-Type', 'text/csv; charset=UTF-8');
+        $response->headers->set('Content-Disposition', 'attachment; filename='. $fileName .'.csv');
+        return $response;
+
+    }
+
+    /**
+     * @param int $deviceId
      * @param ManagerRegistry $managerRegistry
      * @param DeviceRepository $deviceRepository
      * @param ConfigProfileRepository $configProfileRepository
@@ -294,12 +360,20 @@ class DeviceController extends BaseController
      * @throws Exception
      * @throws \JsonException
      */
-    #[Route(path: 'admin/device/{deviceId}', name: 'device_detail')]
-    public function deviceDetail($deviceId, ManagerRegistry $managerRegistry, DeviceRepository $deviceRepository,
-                                 ConfigProfileRepository $configProfileRepository, History $historyService, Command $commandService,
-                                 ParameterBagInterface $params, ApiService $apiService, Request $request,
-                                 DeviceUserModuleRepository $userModuleRepository, MonitoringGroupRepository $monitoringGroupRepository,
-                                 AgentProfiles $agentProfilesApi)
+    #[Route(path: '/{deviceId}', name: '_detail')]
+    public function deviceDetail(
+        int $deviceId,
+        ManagerRegistry $managerRegistry,
+        DeviceRepository $deviceRepository,
+        ConfigProfileRepository $configProfileRepository,
+        History $historyService,
+        Command $commandService,
+        ParameterBagInterface $params,
+        ApiService $apiService,
+        Request $request,
+        DeviceUserModuleRepository $userModuleRepository,
+        MonitoringGroupRepository $monitoringGroupRepository,
+        AgentProfiles $agentProfilesApi)
     {
 
         if (!is_numeric($deviceId)) {
@@ -308,7 +382,7 @@ class DeviceController extends BaseController
 
         }
 
-        /** @var Device $device */
+        /** @var \App\Entity\Device\Device $device */
         $device = $managerRegistry->getRepository(Device::class)->findOneBy(['id' => $deviceId]);
 
         if (!$device || $device->isDeleted()) {
@@ -362,12 +436,7 @@ class DeviceController extends BaseController
         $customModuleDownloadForm = $this->createForm(DeviceCustomModuleDownloadType::class, ['device' => $device]);
         $netDevicesForm = $this->createForm(DeviceNetDevicesType::class, ['netDevices' => $device->getNetDevices()]);
 
-        $customLabels = $managerRegistry->getRepository(Label::class)->findAll();
-        $systemLabels = $managerRegistry->getRepository(SystemLabel::class)->findAll();
-        $deviceLabels = $device->labels;
-        $deviceSystemLabels = $device->systemLabels;
-        $labelsForm = $this->createForm(DeviceCustomLabelType::class, ['deviceLabels' => $deviceLabels, 'labels' => $customLabels]);
-        $systemLabelsForm = $this->createForm(DeviceSystemLabelType::class, ['deviceLabels' => $deviceSystemLabels, 'labels' => $systemLabels]);
+        list($labelsForm, $systemLabelsForm) = $this->getLabelForms($managerRegistry, $device, $request);
 
         $monitoringGroups = $monitoringGroupRepository->getAllMonitoringGroups();
         $deviceMonitoringGroups = $device->monitoringGroupMappings;
@@ -379,7 +448,6 @@ class DeviceController extends BaseController
         $authProfiles = $managerRegistry->getRepository(AuthProfile::class)->findAll();
         $authProfilesForm = $this->createForm(DeviceAuthProfileType::class, ['device' => $device, 'authProfiles' => $authProfiles]);
 
-
         $packages = $userModuleRepository->getDeviceUserModules($device);
 
         $fwVersionForm->handleRequest($request);
@@ -387,8 +455,7 @@ class DeviceController extends BaseController
         $configProfileCloneFromForm->handleRequest($request);
         $configProfileCloneToForm->handleRequest($request);
         $hostnameForm->handleRequest($request);
-        $labelsForm->handleRequest($request);
-        $systemLabelsForm->handleRequest($request);
+
         $monitoringGroupsForm->handleRequest($request);
         $agentProfilesForm->handleRequest($request);
         $authProfilesForm->handleRequest($request);
@@ -404,12 +471,21 @@ class DeviceController extends BaseController
 
         if ($deleteForm->isSubmitted() && $deleteForm->isValid()) {
 
+            $snmpDevice = $managerRegistry->getRepository(SnmpDevice::class);
             if ($apiService->deleteDevice($device->getId()) !== false) {
 
-                $this->addFlash(
-                    'success',
-                    'Device was successfully deleted.'
-                );
+                try{
+                    $snmpDevice->removeSnmpDevice($device->getId());
+                    $this->addFlash(
+                        'success',
+                        'Device was successfully deleted.'
+                    );
+                }catch (\Exception $e){
+                    $this->addFlash(
+                        'danger',
+                        'Device was not deleted.'
+                    );
+                }
 
             } else {
 
@@ -420,24 +496,6 @@ class DeviceController extends BaseController
 
             }
 
-            /*try{
-
-                $deviceRepository->deleteDevice($device->getId());
-
-                $this->addFlash(
-                    'success',
-                    'Device was successfully deleted.'
-                );
-
-            }catch (\Exception $e){
-
-                $this->addFlash(
-                    'danger',
-                    'Device was not deleted.'
-                );
-
-            }*/
-
             return $this->redirectToRoute('admin_devices');
 
         }
@@ -488,7 +546,7 @@ class DeviceController extends BaseController
 
             }
 
-            return $this->redirectToRoute('device_detail', ['deviceId' => $deviceId]);
+            return $this->redirectToRoute('admin_devices_detail', ['deviceId' => $deviceId]);
 
         }
 
@@ -507,290 +565,29 @@ class DeviceController extends BaseController
                 'Hostname will be updated.'
             );
 
-            return $this->redirectToRoute('device_detail', ['deviceId' => $deviceId]);
+            return $this->redirectToRoute('admin_devices_detail', ['deviceId' => $deviceId]);
 
         }
 
         if ($labelsForm->isSubmitted() && $labelsForm->isValid()) {
-
-            $formData = $labelsForm->getData();
-            $deviceLabels = $formData['deviceLabels'];
-            $customLabels = $formData['labels'];
-
-            $deviceLabelsIds = [];
-
-            foreach ($deviceLabels as $label) {
-
-                $deviceLabelsIds[] = $label->getLabel()->getId();
-
-            }
-
-            foreach ($customLabels as $label) {
-
-                //if not exists
-                if (!in_array($label, $deviceLabelsIds)) {
-
-                    $label = $managerRegistry->getRepository(Label::class)->findOneBy(['id' => $label]);
-
-                    $labelDeviceMapping = new LabelDeviceMap();
-                    $labelDeviceMapping->setLabel($label);
-                    $labelDeviceMapping->setDevice($device);
-                    $managerRegistry->getManager()->persist($labelDeviceMapping);
-                    $managerRegistry->getManager()->flush();
-
-                }
-
-                //Remove processed labels from array
-                $key = array_search($label, $deviceLabelsIds);
-                if (false !== $key) {
-                    unset($deviceLabelsIds[$key]);
-                }
-
-            }
-
-            foreach ($deviceLabelsIds as $labelDeleteId) {
-
-                $label = $managerRegistry->getRepository(Label::class)->findOneBy(['id' => $labelDeleteId]);
-                $device = $managerRegistry->getRepository(Device::class)->findOneBy(['id' => $device->getId()]);
-                $labelDeviceMapping = $managerRegistry->getRepository(LabelDeviceMap::class)->findOneBy(
-                    ['label' => $label, 'device' => $device]);
-
-                if ($labelDeviceMapping) {
-
-                    $managerRegistry->getManager()->remove($labelDeviceMapping);
-                    $managerRegistry->getManager()->flush();
-
-                }
-
-            }
-
-            $this->addFlash(
-                'success',
-                'Labels successfully saved.'
-            );
-
-            return $this->redirectToRoute('device_detail', ['deviceId' => $deviceId]);
-
+            return $this->processCustomLabelsFormSubmit($labelsForm, $managerRegistry, $device);
         }
 
         if ($systemLabelsForm->isSubmitted() && $systemLabelsForm->isValid()) {
-
-            $formData = $systemLabelsForm->getData();
-            $deviceLabels = $formData['deviceLabels'];
-            $customLabels = $formData['labels'];
-
-            $deviceLabelsIds = [];
-
-            foreach ($deviceLabels as $label) {
-
-                $deviceLabelsIds[] = $label->getSystemLabel()->getId();
-
-            }
-
-            foreach ($customLabels as $label) {
-
-                //if not exists
-                if (!in_array($label, $deviceLabelsIds)) {
-
-                    $label = $managerRegistry->getRepository(SystemLabel::class)->findOneBy(['id' => $label]);
-
-                    $labelDeviceMapping = new SystemLabelDeviceMap();
-                    $labelDeviceMapping->setSystemLabel($label);
-                    $labelDeviceMapping->setDevice($device);
-                    $managerRegistry->getManager()->persist($labelDeviceMapping);
-                    $managerRegistry->getManager()->flush();
-
-                }
-
-                //Remove processed labels from array
-                $key = array_search($label, $deviceLabelsIds);
-                if (false !== $key) {
-                    unset($deviceLabelsIds[$key]);
-                }
-
-            }
-
-            foreach ($deviceLabelsIds as $labelDeleteId) {
-
-                $label = $managerRegistry->getRepository(SystemLabel::class)->findOneBy(['id' => $labelDeleteId]);
-                $device = $managerRegistry->getRepository(Device::class)->findOneBy(['id' => $device->getId()]);
-                $labelDeviceMapping = $managerRegistry->getRepository(SystemLabelDeviceMap::class)->findOneBy(
-                    ['systemLabel' => $label, 'device' => $device]);
-
-                if ($labelDeviceMapping) {
-
-                    $managerRegistry->getManager()->remove($labelDeviceMapping);
-                    $managerRegistry->getManager()->flush();
-
-                }
-
-            }
-
-            $this->addFlash(
-                'success',
-                'System Labels successfully saved.'
-            );
-
-            return $this->redirectToRoute('device_detail', ['deviceId' => $deviceId]);
-
+            return $this->processSystemLabelsFormSubmit($systemLabelsForm, $managerRegistry, $device);
         }
 
         if ($netDevicesForm->isSubmitted() && $netDevicesForm->isValid()) {
-
-            $data = $netDevicesForm->getData();
-
-            try{
-
-                foreach ($device->getNetDevices() as $netDevice){
-
-                    $formData = $data['cellular_' . $netDevice->getId()];
-                    if($netDevice->isCellular !== $formData){
-
-                        $netDevice->setIsCellular($formData);
-                        $managerRegistry->getManager()->persist($netDevice);
-                        $managerRegistry->getManager()->flush();
-
-                    }
-
-                    $formData = $data['linkable_' . $netDevice->getId()];
-                    if($netDevice->isLinkable !== $formData){
-
-                        $netDevice->setIsLinkable($formData);
-                        $managerRegistry->getManager()->persist($netDevice);
-                        $managerRegistry->getManager()->flush();
-
-                    }
-
-                }
-
-            }catch (\Exception $e){
-
-                $this->addFlash(
-                    'danger',
-                    'Network devices cannot be updated.'
-                );
-
-                return $this->redirectToRoute('device_detail', ['deviceId' => $deviceId]);
-
-            }
-
-            $this->addFlash(
-                'success',
-                'Network devices settings was successfully updated.'
-            );
-
-            return $this->redirectToRoute('device_detail', ['deviceId' => $deviceId]);
-
+            return $this->processNetDevicesFormSubmit($netDevicesForm, $device, $managerRegistry);
         }
 
         if ($monitoringGroupsForm->isSubmitted() && $monitoringGroupsForm->isValid()) {
-
-            $formData = $monitoringGroupsForm->getData();
-            $deviceMonitoringGroups = $formData['deviceMonitoringGroups'];
-            $monitoringGroups = $formData['monitoringGroups'];
-
-            $deviceMonitoringGroupsIds = [];
-
-            foreach ($deviceMonitoringGroups as $monitoringGroup) {
-
-                if($monitoringGroup->getMonitoringGroup()->getId() === 1)
-                    continue;
-                $deviceMonitoringGroupsIds[] = $monitoringGroup->getMonitoringGroup()->getId();
-
-            }
-
-            foreach ($monitoringGroups as $monitoringGroup) {
-
-                //if not exists
-                if (!in_array($monitoringGroup, $deviceMonitoringGroupsIds)) {
-
-                    $monitoringGroup = $managerRegistry->getRepository(MonitoringGroup::class)->findOneBy(['id' => $monitoringGroup]);
-
-                    $monitoringGroupDeviceMapping = new MonitoringGroupMapping();
-                    $monitoringGroupDeviceMapping->setMonitoringGroup($monitoringGroup);
-                    $monitoringGroupDeviceMapping->setDevice($device);
-                    $managerRegistry->getManager()->persist($monitoringGroupDeviceMapping);
-                    $managerRegistry->getManager()->flush();
-
-                }
-
-                //Remove processed labels from array
-                $key = array_search($monitoringGroup, $deviceMonitoringGroupsIds);
-                if (false !== $key) {
-                    unset($deviceMonitoringGroupsIds[$key]);
-                }
-
-            }
-
-            foreach ($deviceMonitoringGroupsIds as $monitoringGroupDeleteId) {
-
-                $monitoringGroup = $managerRegistry->getRepository(MonitoringGroup::class)->findOneBy(['id' => $monitoringGroupDeleteId]);
-                $device = $managerRegistry->getRepository(Device::class)->findOneBy(['id' => $device->getId()]);
-                $monitoringGroupDeviceMapping = $managerRegistry->getRepository(MonitoringGroupMapping::class)->findOneBy(
-                    ['monitoringGroup' => $monitoringGroup, 'device' => $device]);
-
-                if ($monitoringGroupDeviceMapping) {
-
-                    $managerRegistry->getManager()->remove($monitoringGroupDeviceMapping);
-                    $managerRegistry->getManager()->flush();
-
-                }
-
-            }
-
-            $this->addFlash(
-                'success',
-                'Monitoring groups successfully saved.'
-            );
-
-            return $this->redirectToRoute('device_detail', ['deviceId' => $deviceId]);
-
+            return $this->processMonitoringGroupsFormSubmit($monitoringGroupsForm, $managerRegistry, $device);
         }
 
         if ($fwVersionForm->isSubmitted() && $fwVersionForm->isValid()){
+            return $this->processFwVersionFormSubmit($fwVersionForm, $device, $managerRegistry);
 
-            // $form->getData() holds the submitted values
-            // but, the original `$task` variable has also been updated
-            $deviceFwForm = $fwVersionForm->getData();
-
-            if($fwVersionForm->offsetExists('no_specific') && $fwVersionForm->get('no_specific')->isClicked()){
-
-                $device->setOsVerDesired(null);
-                $managerRegistry->getManager()->persist($device);
-                $managerRegistry->getManager()->flush();
-
-                $this->addFlash(
-                    'success',
-                    'Fw version was set to no specific.'
-                );
-
-                return $this->redirectToRoute('device_detail', ['deviceId' => $deviceId]);
-
-            }else{
-
-                //if($device->getOsVer()->getId() !== $deviceFwForm['fw_version']){
-
-                    $osVersion = $managerRegistry->getRepository(DeviceOsVersion::class)
-                        ->findOneBy(['id' =>  $deviceFwForm['fw_version']]);
-                    //dump($osVersion);
-                    //die();
-
-                    $device->setOsVerDesired($osVersion);
-                    $managerRegistry->getManager()->persist($device);
-                    $managerRegistry->getManager()->flush();
-
-                    $this->addFlash(
-                        'success',
-                        'New Fw version was set.'
-                    );
-
-                    return $this->redirectToRoute('device_detail', ['deviceId' => $deviceId]);
-
-                //}
-
-            }
-
-            return $this->redirectToRoute('device_detail', ['deviceId' => $deviceId]);
         }
 
         if ($agentProfilesForm->isSubmitted() && $agentProfilesForm->isValid()){
@@ -812,7 +609,7 @@ class DeviceController extends BaseController
                 'Agent profile was updated.'
             );
 
-            return $this->redirectToRoute('device_detail', ['deviceId' => $deviceId]);
+            return $this->redirectToRoute('admin_devices_detail', ['deviceId' => $deviceId]);
 
         }
 
@@ -833,7 +630,7 @@ class DeviceController extends BaseController
                 'Auth profile was updated.'
             );
 
-            return $this->redirectToRoute('device_detail', ['deviceId' => $deviceId]);
+            return $this->redirectToRoute('admin_devices_detail', ['deviceId' => $deviceId]);
 
         }
 
@@ -871,7 +668,7 @@ class DeviceController extends BaseController
 
             }
 
-            return $this->redirectToRoute('device_detail', ['deviceId' => $deviceId]);
+            return $this->redirectToRoute('admin_devices_detail', ['deviceId' => $deviceId]);
 
         }
 
@@ -916,7 +713,7 @@ class DeviceController extends BaseController
 
                 }
 
-                return $this->redirectToRoute('device_detail', ['deviceId' => $deviceId]);
+                return $this->redirectToRoute('admin_devices_detail', ['deviceId' => $deviceId]);
 
 
             } else {
@@ -932,7 +729,7 @@ class DeviceController extends BaseController
                         'Device profile is empty and cannot be cloned.'
                     );
 
-                    return $this->redirectToRoute('device_detail', ['deviceId' => $deviceId]);
+                    return $this->redirectToRoute('admin_devices_detail', ['deviceId' => $deviceId]);
 
                 }
 
@@ -1010,12 +807,15 @@ class DeviceController extends BaseController
 
                 }
 
-                return $this->redirectToRoute('device_detail', ['deviceId' => $deviceId]);
+                return $this->redirectToRoute('admin_devices_detail', ['deviceId' => $deviceId]);
 
             }
 
         }
 
+       // dump($device->hasCustomFields());
+       // dd($device->customFieldDevice);
+
         return $this->render('device/detail.html.twig', [
             'device' => $device,
             'hostnameForm' => $hostnameForm->createView(),
@@ -1041,6 +841,7 @@ class DeviceController extends BaseController
 
     /**
      * @param int|null $configProfileId
+     * @param array $devicesId
      * @return bool
      * @throws Exception
      */
@@ -1064,4 +865,322 @@ class DeviceController extends BaseController
 
     }
 
+    /**
+     * @param ManagerRegistry $managerRegistry
+     * @param Device $device
+     * @param Request $request
+     * @return array
+     */
+    private function getLabelForms(ManagerRegistry $managerRegistry, Device $device, Request $request): array
+    {
+        $customLabels = $managerRegistry->getRepository(Label::class)->findAll();
+        $systemLabels = $managerRegistry->getRepository(SystemLabel::class)->findAll();
+        $deviceLabels = $device->labels;
+        $deviceSystemLabels = $device->systemLabels;
+        $labelsForm = $this->createForm(DeviceCustomLabelType::class, ['deviceLabels' => $deviceLabels, 'labels' => $customLabels]);
+        $systemLabelsForm = $this->createForm(DeviceSystemLabelType::class, ['deviceLabels' => $deviceSystemLabels, 'labels' => $systemLabels]);
+        $labelsForm->handleRequest($request);
+        $systemLabelsForm->handleRequest($request);
+        return array($labelsForm, $systemLabelsForm);
+    }
+
+    /**
+     * @param FormInterface $labelsForm
+     * @param ManagerRegistry $managerRegistry
+     * @param Device $device
+     * @return RedirectResponse
+     */
+    private function processCustomLabelsFormSubmit(FormInterface $labelsForm, ManagerRegistry $managerRegistry, Device $device): RedirectResponse
+    {
+        $formData = $labelsForm->getData();
+        $deviceLabels = $formData['deviceLabels'];
+        $customLabels = $formData['labels'];
+
+        $deviceLabelsIds = [];
+
+        foreach ($deviceLabels as $label) {
+
+            $deviceLabelsIds[] = $label->getLabel()->getId();
+
+        }
+
+        foreach ($customLabels as $label) {
+
+            //if not exists
+            if (!in_array($label, $deviceLabelsIds)) {
+
+                $label = $managerRegistry->getRepository(Label::class)->findOneBy(['id' => $label]);
+
+                $labelDeviceMapping = new LabelDeviceMap();
+                $labelDeviceMapping->setLabel($label);
+                $labelDeviceMapping->setDevice($device);
+                $managerRegistry->getManager()->persist($labelDeviceMapping);
+                $managerRegistry->getManager()->flush();
+
+            }
+
+            //Remove processed labels from array
+            $key = array_search($label, $deviceLabelsIds);
+            if (false !== $key) {
+                unset($deviceLabelsIds[$key]);
+            }
+
+        }
+
+        foreach ($deviceLabelsIds as $labelDeleteId) {
+
+            $label = $managerRegistry->getRepository(Label::class)->findOneBy(['id' => $labelDeleteId]);
+            $device = $managerRegistry->getRepository(Device::class)->findOneBy(['id' => $device->getId()]);
+            $labelDeviceMapping = $managerRegistry->getRepository(LabelDeviceMap::class)->findOneBy(
+                ['label' => $label, 'device' => $device]);
+
+            if ($labelDeviceMapping) {
+
+                $managerRegistry->getManager()->remove($labelDeviceMapping);
+                $managerRegistry->getManager()->flush();
+
+            }
+
+        }
+
+        $this->addFlash(
+            'success',
+            'Labels successfully saved.'
+        );
+
+        return $this->redirectToRoute('admin_devices_detail', ['deviceId' => $device->getId()]);
+    }
+
+    /**
+     * @param FormInterface $systemLabelsForm
+     * @param ManagerRegistry $managerRegistry
+     * @param Device $device
+     * @return RedirectResponse
+     */
+    private function processSystemLabelsFormSubmit(FormInterface $systemLabelsForm, ManagerRegistry $managerRegistry, Device $device): RedirectResponse
+    {
+        $formData = $systemLabelsForm->getData();
+        $deviceLabels = $formData['deviceLabels'];
+        $systemLabels = $formData['labels'];
+
+        $deviceLabelsIds = [];
+
+        foreach ($deviceLabels as $label) {
+
+            $deviceLabelsIds[] = $label->getSystemLabel()->getId();
+
+        }
+
+        foreach ($systemLabels as $label) {
+
+            //if not exists
+            if (!in_array($label, $deviceLabelsIds)) {
+
+                $label = $managerRegistry->getRepository(SystemLabel::class)->findOneBy(['id' => $label]);
+
+                $labelDeviceMapping = new SystemLabelDeviceMap();
+                $labelDeviceMapping->setSystemLabel($label);
+                $labelDeviceMapping->setDevice($device);
+                $managerRegistry->getManager()->persist($labelDeviceMapping);
+                $managerRegistry->getManager()->flush();
+
+            }
+
+            //Remove processed labels from array
+            $key = array_search($label, $deviceLabelsIds);
+            if (false !== $key) {
+                unset($deviceLabelsIds[$key]);
+            }
+
+        }
+
+        foreach ($deviceLabelsIds as $labelDeleteId) {
+
+            $label = $managerRegistry->getRepository(SystemLabel::class)->findOneBy(['id' => $labelDeleteId]);
+            $device = $managerRegistry->getRepository(Device::class)->findOneBy(['id' => $device->getId()]);
+            $labelDeviceMapping = $managerRegistry->getRepository(SystemLabelDeviceMap::class)->findOneBy(
+                ['systemLabel' => $label, 'device' => $device]);
+
+            if ($labelDeviceMapping) {
+
+                $managerRegistry->getManager()->remove($labelDeviceMapping);
+                $managerRegistry->getManager()->flush();
+
+            }
+
+        }
+
+        $this->addFlash(
+            'success',
+            'System Labels successfully saved.'
+        );
+
+        return $this->redirectToRoute('admin_devices_detail', ['deviceId' => $device->getId()]);
+    }
+
+    /**
+     * @param FormInterface $netDevicesForm
+     * @param Device $device
+     * @param ManagerRegistry $managerRegistry
+     * @return RedirectResponse
+     */
+    private function processNetDevicesFormSubmit(FormInterface $netDevicesForm, Device $device, ManagerRegistry $managerRegistry): RedirectResponse
+    {
+        $data = $netDevicesForm->getData();
+
+        try {
+
+            foreach ($device->getNetDevices() as $netDevice) {
+
+                $formData = $data['cellular_' . $netDevice->getId()];
+                if ($netDevice->isCellular !== $formData) {
+
+                    $netDevice->setIsCellular($formData);
+                    $managerRegistry->getManager()->persist($netDevice);
+                    $managerRegistry->getManager()->flush();
+
+                }
+
+                $formData = $data['linkable_' . $netDevice->getId()];
+                if ($netDevice->isLinkable !== $formData) {
+
+                    $netDevice->setIsLinkable($formData);
+                    $managerRegistry->getManager()->persist($netDevice);
+                    $managerRegistry->getManager()->flush();
+
+                }
+
+            }
+
+        } catch (\Exception $e) {
+
+            $this->addFlash(
+                'danger',
+                'Network devices cannot be updated.'
+            );
+
+            return $this->redirectToRoute('admin_devices_detail', ['deviceId' => $device->getId()]);
+
+        }
+
+        $this->addFlash(
+            'success',
+            'Network devices settings was successfully updated.'
+        );
+
+        return $this->redirectToRoute('admin_devices_detail', ['deviceId' => $device->getId()]);
+    }
+
+    /**
+     * @param FormInterface $monitoringGroupsForm
+     * @param ManagerRegistry $managerRegistry
+     * @param Device $device
+     * @return RedirectResponse
+     */
+    private function processMonitoringGroupsFormSubmit(FormInterface $monitoringGroupsForm, ManagerRegistry $managerRegistry, Device $device): RedirectResponse
+    {
+        $formData = $monitoringGroupsForm->getData();
+        $deviceMonitoringGroups = $formData['deviceMonitoringGroups'];
+        $monitoringGroups = $formData['monitoringGroups'];
+
+        $deviceMonitoringGroupsIds = [];
+
+        foreach ($deviceMonitoringGroups as $monitoringGroup) {
+
+            if ($monitoringGroup->getMonitoringGroup()->getId() === 1) {
+                continue;
+            }
+            $deviceMonitoringGroupsIds[] = $monitoringGroup->getMonitoringGroup()->getId();
+
+        }
+
+        foreach ($monitoringGroups as $monitoringGroup) {
+
+            //if not exists
+            if (!in_array($monitoringGroup, $deviceMonitoringGroupsIds, true)) {
+
+                $monitoringGroup = $managerRegistry->getRepository(MonitoringGroup::class)->findOneBy(['id' => $monitoringGroup]);
+
+                $monitoringGroupDeviceMapping = new MonitoringGroupMapping();
+                $monitoringGroupDeviceMapping->setMonitoringGroup($monitoringGroup);
+                $monitoringGroupDeviceMapping->setDevice($device);
+                $managerRegistry->getManager()->persist($monitoringGroupDeviceMapping);
+                $managerRegistry->getManager()->flush();
+
+            }
+
+            //Remove processed labels from array
+            $key = array_search($monitoringGroup, $deviceMonitoringGroupsIds);
+            if (false !== $key) {
+                unset($deviceMonitoringGroupsIds[$key]);
+            }
+
+        }
+
+        foreach ($deviceMonitoringGroupsIds as $monitoringGroupDeleteId) {
+
+            $monitoringGroup = $managerRegistry->getRepository(MonitoringGroup::class)->findOneBy(['id' => $monitoringGroupDeleteId]);
+            $device = $managerRegistry->getRepository(Device::class)->findOneBy(['id' => $device->getId()]);
+            $monitoringGroupDeviceMapping = $managerRegistry->getRepository(MonitoringGroupMapping::class)->findOneBy(
+                ['monitoringGroup' => $monitoringGroup, 'device' => $device]);
+
+            if ($monitoringGroupDeviceMapping) {
+
+                $managerRegistry->getManager()->remove($monitoringGroupDeviceMapping);
+                $managerRegistry->getManager()->flush();
+
+            }
+
+        }
+
+        $this->addFlash(
+            'success',
+            'Monitoring groups successfully saved.'
+        );
+
+        return $this->redirectToRoute('admin_devices_detail', ['deviceId' => $device->getId()]);
+    }
+
+    /**
+     * @param FormInterface $fwVersionForm
+     * @param Device $device
+     * @param ManagerRegistry $managerRegistry
+     * @return RedirectResponse
+     */
+    private function processFwVersionFormSubmit(FormInterface $fwVersionForm, Device $device, ManagerRegistry $managerRegistry): RedirectResponse
+    {
+        // $form->getData() holds the submitted values
+        // but, the original `$task` variable has also been updated
+        $deviceFwForm = $fwVersionForm->getData();
+
+        if ($fwVersionForm->offsetExists('no_specific') && $fwVersionForm->get('no_specific')->isClicked()) {
+
+            $device->setOsVerDesired(null);
+            $managerRegistry->getManager()->persist($device);
+            $managerRegistry->getManager()->flush();
+
+            $this->addFlash(
+                'success',
+                'Fw version was set to no specific.'
+            );
+
+            return $this->redirectToRoute('admin_devices_detail', ['deviceId' => $device->getId()]);
+
+        }
+
+        $osVersion = $managerRegistry->getRepository(DeviceOsVersion::class)
+            ->findOneBy(['id' => $deviceFwForm['fw_version']]);
+
+        $device->setOsVerDesired($osVersion);
+        $managerRegistry->getManager()->persist($device);
+        $managerRegistry->getManager()->flush();
+
+        $this->addFlash(
+            'success',
+            'New Fw version was set.'
+        );
+
+        return $this->redirectToRoute('admin_devices_detail', ['deviceId' => $device->getId()]);
+    }
+
 }
\ No newline at end of file
