@@ -4,8 +4,10 @@ declare(strict_types=1);
 
 namespace App\Controller\Admin\System;
 
-use App\Datatables\DeviceDatatable;
+use App\DataTable\DeviceDatatable;
+use App\Form\Settings\DclSettingsType;
 use App\Form\SettingsType;
+use App\Repository\DataAtomRepository;
 use App\Repository\DeviceAgentRepository;
 use App\Repository\ProductFlagsRepository;
 use App\Repository\TypesValueTypeRepository;
@@ -37,12 +39,15 @@ class SettingsController extends Controller {
     {
     }
 
-    /**
-     * @return RedirectResponse|Response
-     * @throws \Exception
-     */
+	/**
+	 * @param Request $request
+	 * @param ConfigurationService $configurationService
+	 * @param Command $commandService
+	 * @return RedirectResponse|Response
+	 */
     #[Route(path: '/admin/settings', name: 'admin_settings')]
-    public function default(Request $request, ConfigurationService $configurationService, Command $commandService) {
+    public function default(Request $request, ConfigurationService $configurationService, Command $commandService): RedirectResponse|Response
+	{
 
         $settings = $this->getSettings($configurationService);
 
@@ -63,7 +68,7 @@ class SettingsController extends Controller {
         $settingsForm->handleRequest($request);
 
         if ($settingsForm->isSubmitted() && $settingsForm->isValid()) {
-            $inputDataFormNames = array_map(fn($ar) => ($ar['writable'] === true) ? $ar['name'] : null, $configFormData);
+            $inputDataFormNames = array_map(static fn($ar) => ($ar['writable'] === true) ? $ar['name'] : null, $configFormData);
             $formData = $settingsForm->getData();
 
             foreach ($inputDataFormNames as $key => $inputName) {
@@ -123,25 +128,78 @@ class SettingsController extends Controller {
             }
         }
 
-		$configNames = [];
-		//Get group builder which is part of the form named 'config'
-		foreach ($settingsForm->all()['config'] as $item){
-			//Used to generate js to disable and enable items in the form
-			$configNames[] = $item->getName();
-		}
-
-		return $this->render('system/settings.html.twig', [
-			'settingsForm' => $settingsForm->createView(),
-			'configNames' => $configNames ?? null,
-			'nonWritableConfigData' => $this->getNonWritableConfigData($configFormData),
-			'deviceTableColumnsChoices' => $deviceTableColumnsChoices,
-		]);
+        return $this->render('system/settings.html.twig', [
+            'settingsForm' => $settingsForm->createView(),
+            //Get group builder which is part of the form named 'config'
+            'configItems' => $settingsForm->all()['config'] ?? null,
+            'nonWritableConfigData' => $this->getNonWritableConfigData($configFormData),
+            'deviceTableColumnsChoices' => $deviceTableColumnsChoices,
+        ]);
     }
 
-    /**
-     * @return Response
-     * @throws \Exception
-     */
+	#[Route(path: '/admin/settings/dcl', name: 'admin_settings_dcl_atom')]
+	public function dclAtomConfiguration(Request $request, DataAtomRepository $dataAtomRepository): Response
+	{
+
+		$atomTypes = $dataAtomRepository->findBy(['visible' => true]);
+		$settingsForm = $this->createForm(DclSettingsType::class, ['configFormData' => $atomTypes]);
+
+		$settingsForm->handleRequest($request);
+
+		if ($settingsForm->isSubmitted() && $settingsForm->isValid()) {
+
+			$formData =  $settingsForm->getData();
+			//we do not want these data for iteration
+			unset($formData['configFormData']);
+			$updateData = [];
+			foreach ($formData as $name => $value) {
+
+				$items = explode('_', $name);
+				$updateData[$items[0]][$items[1]] = $value;
+
+			}
+
+			foreach ($updateData as $id => $values) {
+				$result = $dataAtomRepository->update($id, $values);
+
+			}
+
+			if($result){
+
+				$this->addFlash(
+					'success',
+					'Data was updated.',
+				);
+
+			}else{
+
+				$this->addFlash(
+					'danger',
+					'Cannot update data.' ,
+				);
+
+			}
+
+			return $this->redirectToRoute('admin_settings_dcl_atom');
+
+		}
+
+		return $this->render('system/settings/dcl.html.twig', [
+			'settingsForm' => $settingsForm->createView(),
+			//Get group builder which is part of the form named 'config'
+			'configItems' => $settingsForm->all()['config'] ?? null,
+		]);
+
+	}
+
+	/**
+	 * @param Request $request
+	 * @param ConfigurationService $configurationService
+	 * @param ProductFlagsRepository $productFlagsRepository
+	 * @param DeviceAgentRepository $deviceAgentRepository
+	 * @return Response
+	 * @throws \Exception
+	 */
     #[Route(path: '/admin/device-agents', name: 'admin_device_agents')]
     public function deviceAgents(
         Request                $request,
@@ -288,27 +346,34 @@ class SettingsController extends Controller {
             'isSlackSettingsComplete' => $isSlackSettingsComplete,
             'isEmailSettingsComplete' => $isEmailSettingsComplete,
             'deviceTableColumns' => $deviceTableColumns,
+            'snmpGatewayThreads' => $configurationService->getSnmpGatewayThreads(),
+            'snmpGatewayBatch' => $configurationService->getSnmpGatewayBatch(),
+            'snmpGatewayTimeout' => $configurationService->getSnmpGatewayTimeout(),
+            'snmpGatewayRetries' => $configurationService->getSnmpGatewayRetries(),
+            'snmpGatewayInterval' => $configurationService->getSnmpGatewayInterval(),
+            'snmpLogLevel' => $configurationService->getSnmpLogLevel(),
+            'snmpSslOverride' => $configurationService->getSnmpSslOverride(),
             'configFormData' => $configFormData,
         ];
 
     }
 
-    /**
-     * @return array
-     */
+	/**
+	 * @param array $configFormData
+	 * @return array
+	 */
     private function getNonWritableConfigData(array $configFormData): array {
 
         return array_filter($configFormData, fn($v) => $v['writable'] === false);
 
     }
 
-    /**
-     * @param FormInterface $settingsForm
-     * @param ConfigurationService $configurationService
-     * @param $serverIpAddress
-     * @param Command $commandService
-     * @return RedirectResponse
-     */
+	/**
+	 * @param FormInterface $settingsForm
+	 * @param ConfigurationService $configurationService
+	 * @param $serverIpAddress
+	 * @return RedirectResponse
+	 */
     private function saveSettings(FormInterface $settingsForm, ConfigurationService $configurationService, $serverIpAddress): RedirectResponse
     {
         $serverSettings = $settingsForm->getData();
@@ -324,6 +389,13 @@ class SettingsController extends Controller {
         $configurationService->updateSmtpPassword($serverSettings['smtpPassword']);
         $configurationService->updateSlackToken($serverSettings['slackToken']);
         $configurationService->updateSlackChannel($serverSettings['slackChannel']);
+        $configurationService->updateSnmpGatewayThreads($serverSettings['snmpGatewayThreads']);
+        $configurationService->updateSnmpGatewayBatch($serverSettings['snmpGatewayBatch']);
+        $configurationService->updateSnmpGatewayTimeout($serverSettings['snmpGatewayTimeout']);
+        $configurationService->updateSnmpGatewayRetries($serverSettings['snmpGatewayRetries']);
+        $configurationService->updateSnmpGatewayInterval($serverSettings['snmpGatewayInterval']);
+        $configurationService->updateSnmpLogLevel($serverSettings['snmpLogLevel']);
+        $configurationService->updateSnmpSslOverride($serverSettings['snmpSslOverride']);
 
         if ($serverSettings['serverIpAddress'] !== $serverIpAddress) {
             // Update Server IP Address
