@@ -2,7 +2,7 @@
 
 namespace App\Form\Device;
 
-use App\Entity\Device;
+use App\Entity\Device\Device;
 use Symfony\Component\Form\AbstractType;
 use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
 use Symfony\Component\Form\Extension\Core\Type\SubmitType;
