@@ -2,7 +2,7 @@
 
 namespace App\Form\Device;
 
-use App\Entity\Label;
+use App\Entity\Label\Label;
 use Symfony\Component\Form\AbstractType;
 use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
 use Symfony\Component\Form\Extension\Core\Type\SubmitType;
@@ -15,7 +15,8 @@ class DeviceSystemLabelType extends AbstractType
      * @param FormBuilderInterface $builder
      * @param array $options
      */
-    public function buildForm(FormBuilderInterface $builder, array $options){
+    public function buildForm(FormBuilderInterface $builder, array $options): void
+	{
 
         /** @var Label $deviceLabels */
         $deviceLabels = $options['data']['deviceLabels'];
@@ -36,16 +37,19 @@ class DeviceSystemLabelType extends AbstractType
         }
 
         $builder
-            ->add('labels', ChoiceType::class ,array(
+            ->add('labels', ChoiceType::class, [
                 'choices' => $choices,
                 'placeholder' => 'Select a Device to clone from',
                 'multiple' => true,
                 'required' => false,
                 'data' => $selectedItems,
+            ])
 
-            ))
-            ->add('save', SubmitType::class, ['label' => 'Set ',
-                'block_prefix' => 'modal_footer'])
+            ->add('save', SubmitType::class, [
+				'label' => '<i class="fas fa fa-ticket"></i> Set',
+				'label_html' => true,
+                'block_prefix' => 'modal_footer'
+			])
         ;
 
     }
