@@ -2,7 +2,7 @@
 
 namespace App\Repository;
 
-use App\Entity\MonitoringGroup;
+use App\Entity\MonitoringGroup\MonitoringGroup;
 use Doctrine\Bundle\DoctrineBundle\Repository\ServiceEntityRepository;
 use Doctrine\Persistence\ManagerRegistry;
 
